from argparse import REMAINDER, ArgumentParser
from typing import Any, Callable, List, Optional

from .command import (
    handle_commit,
    handle_hook,
    handle_list,
    handle_no_subcommand,
    handle_search,
)

parser = ArgumentParser("soremoji", description="Simple gitmoji cli written in python")
parser.set_defaults(func=handle_no_subcommand)
_subparsers = parser.add_subparsers(title="subcommands")

# hook
hook_parser = _subparsers.add_parser("hook", help="Run as a git hook.")
hook_parser.add_argument("commit_msg_file", help="Path to the commit message file")
hook_parser.set_defaults(func=handle_hook)

# commit
commit_parser = _subparsers.add_parser("commit", help="Start to commit changes.")
commit_parser.add_argument("-e", "--emoji", help="Emoji to use")
commit_parser.add_argument("-m", "--message", help="Commit message")
commit_parser.add_argument(
    "--", nargs=REMAINDER, help="Argument to pass to the git", dest="args"
)
commit_parser.set_defaults(func=handle_commit)

# list
list_parser = _subparsers.add_parser("list", help="List all available emoji.")
list_parser.set_defaults(func=handle_list)

# search
search_parser = _subparsers.add_parser("search", help="Search for an emoji.")
search_parser.set_defaults(func=handle_search)


def main(argv: Optional[List[str]] = None):
    result = parser.parse_args(argv)
    result = vars(result)
    func: Callable[..., Any] = result.pop("func")
    func(**result)



import contextlib
import shlex
import subprocess
from pathlib import Path
from typing import Callable, List, Optional

from sorampt import CancelledError, Choice, ListPrompt

from .emoji import EMOJIS
from .feature import prompt, prompt_emoji


def handle_no_subcommand(**_):
    with contextlib.suppress(CancelledError):
        result = ListPrompt[Callable[..., None]](
            "What do you want to do?",
            [
                Choice("Start to commit changes", handle_commit),
                Choice("List all available emojis", handle_list),
                Choice("Search for an emoji", handle_search),
            ],
        ).prompt()
        result.data()


def handle_hook(commit_msg_file: str, **_):
    file = Path(commit_msg_file)
    original_msg = file.read_text(encoding="utf-8")
    new_msg = prompt(original_msg)
    file.write_text(new_msg, encoding="utf-8")


def handle_commit(
    emoji: Optional[str] = None,
    message: Optional[str] = None,
    args: Optional[List[str]] = None,
    **_,
):
    args = args or []
    new_msg = prompt(message, emoji=emoji)
    pargs = ["git", "commit", "-m", new_msg, *args]
    try:
        subprocess.run(pargs, check=True)
    except Exception as e:
        print("Oops! An error ocurred while committing: ", e)
        print("You can run the same commit with this command:")
        print(f"\t{' '.join(shlex.quote(arg) for arg in pargs)}")


def handle_list(**_):
    text = "\n".join(emoji.to_string() for emoji in EMOJIS.values())
    print(text)


def handle_search(**_):
    print(prompt_emoji())



import contextlib
import re
from dataclasses import dataclass
from typing import Optional, cast

from sorampt import Choice, InputPrompt, ListPrompt

from .emoji import EMOJIS, Emoji

MESSAGE_REGEX = re.compile(r"^:(?P<emoji>\w+):(?P<message>[\s\S]+)$")


@dataclass
class Message:
    emoji: Optional[Emoji] = None
    content: Optional[str] = None

    @property
    def valid(self) -> bool:
        return bool(self.emoji and self.content)

    def to_string(self) -> str:
        if not self.valid:
            raise ValueError("Message is not valid")
        emoji = cast(Emoji, self.emoji)
        content = cast(str, self.content)
        return f"{emoji.code} {content.strip()}"


def transform(message: str) -> Message:
    match = MESSAGE_REGEX.match(message)
    if match:
        return Message(EMOJIS.get(match.group("emoji")), match.group("message"))
    return Message(None, message)


def prompt_emoji() -> Emoji:
    return (
        ListPrompt(
            "Choose a gitmoji:",
            [
                Choice(
                    emoji.to_string(),
                    emoji,
                )
                for emoji in EMOJIS.values()
            ],
        )
        .prompt()
        .data
    )


def prompt(original_msg: Optional[str] = None, emoji: Optional[str] = None) -> str:
    if emoji:
        message = Message(EMOJIS.get(emoji), original_msg)
    elif original_msg:
        message = transform(original_msg)
    else:
        message = Message()

    with contextlib.suppress(Exception):
        if not message.emoji:
            message.emoji = prompt_emoji()

        message.content = InputPrompt(
            "Enter the commit title:",
            default_text=message.content,
            validator=lambda x: bool(x.strip()),
        ).prompt()

    return message.to_string()



from dataclasses import dataclass

from wcwidth import wcswidth


@dataclass(frozen=True)
class Emoji:
    name: str
    emoji: str
    description: str

    @property
    def code(self) -> str:
        return f":{self.name}:"

    def to_string(self) -> str:
        return f"{self.emoji}{' ' * (4 - wcswidth(self.emoji))}- {self.description}"


EMOJIS = {
    emoji["name"]: Emoji(**emoji)
    for emoji in [
        {
            "emoji": "✨",
            "description": "Introduce new features.",
            "name": "sparkles",
        },
        {
            "emoji": "🐛",
            "description": "Fix a bug.",
            "name": "bug",
        },
        {
            "emoji": "📝",
            "description": "Add or update documentation.",
            "name": "memo",
        },
        {
            "emoji": "📄",
            "description": "Add or update license.",
            "name": "page_facing_up",
        },
        {
            "emoji": "🎨",
            "description": "Improve structure / format of the code.",
            "name": "art",
        },
        {
            "emoji": "♻️",
            "description": "Refactor code.",
            "name": "recycle",
        },
        {
            "emoji": "✅",
            "description": "Add, update, or pass tests.",
            "name": "white_check_mark",
        },
        {
            "emoji": "🚀",
            "description": "Deploy stuff.",
            "name": "rocket",
        },
        {
            "emoji": "💄",
            "description": "Add or update the UI and style files.",
            "name": "lipstick",
        },
        {
            "emoji": "🔖",
            "description": "Release / Version tags.",
            "name": "bookmark",
        },
        {
            "emoji": "🏷️",
            "description": "Add or update types.",
            "name": "label",
        },
        {
            "emoji": "🚨",
            "description": "Fix compiler / linter warnings.",
            "name": "rotating_light",
        },
        {
            "emoji": "➕",
            "description": "Add a dependency.",
            "name": "heavy_plus_sign",
        },
        {
            "emoji": "➖",
            "description": "Remove a dependency.",
            "name": "heavy_minus_sign",
        },
        {
            "emoji": "⬇️",
            "description": "Downgrade dependencies.",
            "name": "arrow_down",
        },
        {
            "emoji": "⬆️",
            "description": "Upgrade dependencies.",
            "name": "arrow_up",
        },
        {
            "emoji": "📌",
            "description": "Pin dependencies to specific versions.",
            "name": "pushpin",
        },
        {
            "emoji": "💚",
            "description": "Fix CI Build.",
            "name": "green_heart",
        },
        {
            "emoji": "👷",
            "description": "Add or update CI build system.",
            "name": "construction_worker",
        },
        {
            "emoji": "📈",
            "description": "Add or update analytics or track code.",
            "name": "chart_with_upwards_trend",
        },
        {
            "emoji": "🔧",
            "description": "Add or update configuration files.",
            "name": "wrench",
        },
        {
            "emoji": "🔨",
            "description": "Add or update development scripts.",
            "name": "hammer",
        },
        {
            "emoji": "🌐",
            "description": "Internationalization and localization.",
            "name": "globe_with_meridians",
        },
        {
            "emoji": "✏️",
            "description": "Fix typos.",
            "name": "pencil2",
        },
        {
            "emoji": "🎉",
            "description": "Begin a project.",
            "name": "tada",
        },
        {
            "emoji": "🚧",
            "description": "Work in progress.",
            "name": "construction",
        },
        {
            "emoji": "⏪️",
            "description": "Revert changes.",
            "name": "rewind",
        },
        {
            "emoji": "🔀",
            "description": "Merge branches.",
            "name": "twisted_rightwards_arrows",
        },
        {
            "emoji": "📦️",
            "description": "Add or update compiled files or packages.",
            "name": "package",
        },
        {
            "emoji": "👽️",
            "description": "Update code due to external API changes.",
            "name": "alien",
        },
        {
            "emoji": "🚚",
            "description": "Move or rename resources (e.g.: files, paths, routes).",
            "name": "truck",
        },
        {
            "emoji": "💥",
            "description": "Introduce breaking changes.",
            "name": "boom",
        },
        {
            "emoji": "♿️",
            "description": "Improve accessibility.",
            "name": "wheelchair",
        },
        {
            "emoji": "💡",
            "description": "Add or update comments in source code.",
            "name": "bulb",
        },
        {
            "emoji": "🍻",
            "description": "Write code drunkenly.",
            "name": "beers",
        },
        {
            "emoji": "🔊",
            "description": "Add or update logs.",
            "name": "loud_sound",
        },
        {
            "emoji": "🔇",
            "description": "Remove logs.",
            "name": "mute",
        },
        {
            "emoji": "🚸",
            "description": "Improve user experience / usability.",
            "name": "children_crossing",
        },
        {
            "emoji": "🏗️",
            "description": "Make architectural changes.",
            "name": "building_construction",
        },
        {
            "emoji": "🙈",
            "description": "Add or update a .gitignore file.",
            "name": "see_no_evil",
        },
        {
            "emoji": "🚩",
            "description": "Add, update, or remove feature flags.",
            "name": "triangular_flag_on_post",
        },
        {
            "emoji": "🥅",
            "description": "Catch errors.",
            "name": "goal_net",
        },
        {
            "emoji": "🗑️",
            "description": "Deprecate code that needs to be cleaned up.",
            "name": "wastebasket",
        },
        {
            "emoji": "⚰️",
            "description": "Remove dead code.",
            "name": "coffin",
        },
        {
            "emoji": "🧑‍💻",
            "description": "Improve developer experience",
            "name": "technologist",
        },
        {
            "emoji": "⚡️",
            "description": "Improve performance.",
            "name": "zap",
        },
        {
            "emoji": "🔥",
            "description": "Remove code or files.",
            "name": "fire",
        },
        {
            "emoji": "🍱",
            "description": "Add or update assets.",
            "name": "bento",
        },
    ]
}
